package CheckRelease::Util;

use strict;
use warnings;

use base 'Exporter';
our @EXPORT_OK = qw( get_permissions get_config_value get_config_hash _scope
    want_cross_check get_checkers is_editor check_url button_html mail_head is_administer
    update_future_que load_latest_revision send_notice_mail cf_meta load_checkers
    get_crChangeBtnLabel unable_plugin set_param_session get_param_session );

use MT::App::CMS;
use MT::Util qw(format_ts is_valid_email);
use Ideamans::Util qw(array_search);

sub get_permissions {
    my $app = MT::App::CMS->instance;
    my $blog_id = $app->param('blog_id') || 0;
    my @permissions;

    # すべての記事の編集権限を持つかどうかで、承認者とする
    foreach ( qw/edit_all_posts/ ) {
        # プラグインデータ未保存だと、is_editor内でパーミッションチェックがされないため、
        # ブログ記事の一覧でリンクが非表示になってしまう
        push @permissions, $_; # if get_config_value($_);
    }
    \@permissions;
}

sub get_config_value {
    my ($key, $scope) = @_;
    my $plugin = MT->component('CheckRelease');
    my $config = $plugin->get_config_hash( $scope ? $scope : _scope() );
    return $config->{$key} if exists($config->{$key});
    $plugin->get_config_value($key, 'system');
}

sub get_config_hash {
    my ($scope) = @_;
    my $plugin = MT->component('CheckRelease');
    my $config = $plugin->get_config_hash( $scope ? $scope : _scope() );
    return $config if %$config;
    $plugin->get_config_hash();
}

sub want_cross_check {
    my ($user) = @_;
    my $config = get_config_hash();
    (defined($config->{checker_cross_check})
        && $config->{checker_cross_check})
        ? 1 : 0;
}

sub get_checkers {
    my ($terms, $args) = @_;
    $terms = {} unless ref($terms) eq 'HASH';
    $args = {} unless ref($args) eq 'HASH';
    my $app = MT::App::CMS->instance;
    if ( !%$terms && !%$args ) {
        my $blog_id = $app->param('blog_id') || 0;
        my $permissions = get_permissions();
        my $term;
        $term->{blog_id} = [ $blog_id, 0 ];
        $term->{permissions} = [];
        foreach ( @$permissions ) {
            my $cond = { like => "%'$_'%" };
            if ( @{ $term->{permissions} } ) {
                push @{ $term->{permissions} }, -or => $cond;
            } else {
                push @{ $term->{permissions} }, $cond;
            }
            # ウェブサイト管理者権限の検索はウェブサイトのblog_idが必要
            if ( $_ eq 'administer_website' ) {
                my $blog = $app->model('blog')->load($blog_id);
                my $website_id = $blog->parent_id;
                push @{ $term->{blog_id} }, $website_id;
            }
        }
        $args->{'join'} = [
            'MT::Permission',
            'author_id',
            $term,
            { unique => 1, 'sort' => 'author_id' } ];
    }
    my @authors = $app->model('author')->load($terms, $args);
    my @actives = grep { $_->status == MT::Author::ACTIVE() } map { $_ } @authors;

    \@actives;
}

sub _scope {
    my $app = MT::App::CMS->instance;
    my $blog_id = $app->param('blog_id');
    $blog_id ? "blog:$blog_id" : 'system';
}

sub is_editor {
    my ( $user ) = @_;
    return 1 if $user->is_superuser;
    my $permissions = get_permissions();
    my $is_editor = 0;
    require MT::App::CMS;
    my $app = MT::App::CMS->instance;
    my $perms = $user->permissions( $app->param('blog_id') );
    foreach ( @$permissions ) {
        my $can_do = 'can_' .$_;
        $is_editor = 1 if $perms->$can_do();
    }
    $is_editor;
}


sub check_url {
    my ($app, $obj, $mode, $cf_meta) = @_;
    $cf_meta = {} unless defined $cf_meta;
    my $plugin = $app->component('CheckRelease');
    my $args;
    my $columns = $obj->column_names;
    my $blog = $obj->blog;

    my %config;
    $plugin->load_config(\%config, 'system');

    if ( $mode eq 'preview_entry' ) {
        foreach ( @$columns ) {
            if ( $_ eq 'authored_on' || $_ eq 'created_on' || $_ eq 'modified_on' ) {
                $args->{$_ .'_date'} = format_ts('%Y-%m-%d', $obj->$_, $blog, 'en', 0);
                $args->{$_ .'_time'} = format_ts('%H:%M:%S', $obj->$_, $blog, 'en', 0);
            }
            else {
                $args->{$_} = $obj->$_;
            }
        }
        foreach ( qw/include_asset_ids category_ids tags/ ) {
            $args->{$_} = $app->param($_);
        }

        # カスタムフィールド対応
        foreach ( keys %$cf_meta ) {
            my $cf = "customfield_$_";
            $args->{$cf} = $cf_meta->{$_};
            if ( $args->{$cf} =~ /^\d{14}$/ ) {
                $args->{"d_$cf"} = format_ts('%Y-%m-%d', $cf_meta->{$_}, $obj->blog, 'en', 0);
                $args->{"t_$cf"} = format_ts('%H:%M:%S', $cf_meta->{$_}, $obj->blog, 'en', 0);
            }
        }
        $args->{customfield_beacon} = 1;
        my %params = $app->param_hash;
        foreach ( keys %params ) {
            $args->{$_} ||= $params{$_};
        }
    }
    else {
        $args->{$_} = $app->param($_) foreach qw/id _type blog_id r crr/;
        $args->{id} = $obj->id;
    }
    $args->{r} = $app->param('r') if $app->param('r');
    $args->{r} = $app->param('pre_r') if $app->param('pre_r');
    $args->{crr} = $app->param('crr') if $app->param('crr');
    $args->{'checkrelease_checkers[]'} = [ split /\s*,\s*/, $obj->check_requested_authors ];

    # リビジョン新規作成の場合は最新のリビジョン番号を取得
    if ( defined($args->{crr})
        && $args->{crr}
        && !( defined($args->{r}) && $args->{r} ) ) {
        my $rev = load_latest_revision($app, $obj->class, $obj->id);
        $args->{r} = ref($rev) ? $rev->rev_number : 1;
    }

    # セッションパラメータによりURLを短縮
    my $blog_id = delete $args->{blog_id};
    my ( $token, $salt ) = set_param_session( $app->config('CRPreviewURLExpires'), $args );
    $args = { cr_token => $token, cr_salt => $salt };
    $args->{blog_id} = $blog_id if defined $blog_id;

    my $uri = $app->config->CGIPath;
    if ( defined($config{use_ssl_url}) && $config{use_ssl_url} =~ m!^https?://! ) {
        my $admin_host = $config{use_ssl_url};
        $admin_host .= '/' unless $admin_host =~ m!/$!;
        $uri = $admin_host .'mt/admin';
    } else {
        $uri .= $app->config->AdminScript;
    }

    $uri .= $app->uri_params(mode => $mode, args => $args);
    unless ( $uri =~ /^https?/ ) {
        my $blog_url = $blog->site_url;
        $blog_url .= '/' unless $blog_url =~ m!/$!;
        my ($site_url) = ($blog_url =~ m!^(https?://[^/]+)/!);
        $uri = "$site_url$uri";
    }
    $uri =~ s/^http:/https:/ if $config{use_ssl_url} == 1;
    $uri;
}

sub set_param_session {
    my( $expires, $param ) = @_;

    my $t = time;
    $expires = int($expires);

    my $salt = int(rand(1000000));
    require Digest::MD5;
    my $token = Digest::MD5::md5_hex(time + $salt);

    require MT::Session;
    my $sess = MT::Session->new;
    $sess->set_values({
        kind        => '_C',
        id          => $token,
        name        => $salt,
        start       => $t,
        duration    => $t + $expires,
    });
    $sess->set('param', $param);
    $sess->save;

    ( $token, $salt );
}

sub get_param_session {
    my ( $token, $salt ) = @_;

    require MT::Session;
    my $sess = MT::Session->load({
        kind        => '_C',
        id          => $token,
        name        => $salt,
        duration    => [ time, undef ],
    }, {
        range       => { duration => 1 },
    });

    return unless $sess;

    my $param = $sess->get('param');
    $param;
}

sub load_latest_revision {
    my ($app, $class, $id) = @_;
    my $pkg = $app->model("$class:revision");
    return undef unless $pkg;
    my $terms;
    $terms->{$class .'_id'} = $id;
    my $args;
    $args->{limit} = 1;
    $args->{'sort'} = 'rev_number';
    $args->{direction} = 'descend';
    my @revs = $pkg->load($terms, $args);
    @revs ? shift @revs : undef;
}

sub button_html {
    {
        draft => q{<__trans_section component="CheckRelease"><button mt:mode="save_entry" type="submit" name="check_status" title="<__trans phrase="Return to">" class="check_status action" value="1"><__trans phrase="Return to(Draft)"></button></__trans_section>},
        publish => q{<__trans_section component="CheckRelease"><button mt:mode="save_entry" type="submit" name="check_status" title="<__trans phrase="Return to">" class="check_status action" value="1"><__trans phrase="Return to(As Publish)"></button></__trans_section>},
        none => '',
    }->{$_[0]};
}

sub mail_head {
    my ($app, $subject, $user_email, $emails) = @_;
    my $config = $app->config;
    my $from = $config->EmailAddressMain;
    my $reply_to = $config->EmailReplyTo || $from;
    $from = undef if $from && !is_valid_email($from);
    $reply_to = undef if $reply_to && !is_valid_email($reply_to);
    $emails = [] unless ref($emails) eq 'ARRAY';
    unless ( $from || $reply_to ) {
        $app->log( MT->translate('System Email Address is not configured.') );
        return;
    }
    my %head;
    $head{Subject} = $subject;
    $head{From} = $from if $from;
    $head{'Reply-To'} = $reply_to if $reply_to;
    my $charset = $config->MailEncoding || $config->PublishCharset;
    $head{'Content-Type'} = qq(text/plain; charset="$charset");
    $head{Bcc} = [];
    if ( @$emails ) {
        foreach ( @$emails ) {
            push @{ $head{Bcc} }, $_ unless $user_email eq $_;
        }
    }
    %head;
}

sub is_administer {
    is_editor(@_);
}

sub update_future_que {
    my ($app, $rev_obj, $datasource) = @_;
    my $plugin = $app->component('CheckRelease');
    my $key = 'futurepost_que';
    my $name = 'CheckRelease::Plugin';
    my $terms;
    $terms->{plugin} = $name;
    $terms->{key} = $key;
    require MT::PluginData;
    my $pd = MT::PluginData->load($terms);
    unless ( ref $pd ) {
        $pd = MT::PluginData->new;
        $pd->key( $key );
        $pd->plugin( $name );
    }
    my $data = $pd->data();
    my $obj_id = $datasource .'_id';
    my $id = $rev_obj->$obj_id;
    require MT::Entry;
    if ( (MT->version_number < 5.1 && $app->param('scheduled'))
    	|| $app->param('status') == MT::Entry::FUTURE() ) {
        $data->{$id} = $rev_obj->id;
    }
    else {
        delete $data->{$id} if defined $data->{$obj_id};
    }
    $pd->data( $data );
    $pd->save()
        or die $pd->errstr;
}

sub send_notice_mail {
    my ($app, $obj, $author, $new_revision) = @_;
    my $user = $app->user;
    my $plugin = $app->component('CheckRelease');

    # 承認者情報をemailでまとめる
    my $terms;
    $terms->{id} = [ split /\s*,\s*/, $obj->check_requested_authors ];
    my $checkers = get_checkers($terms);
    my %emails;
    foreach ( @$checkers ) {
        next if defined $emails{$_->email};
        $emails{ $_->email } = $_;
    }

    # 投稿者に通知
    $author = $obj->author unless ref $author;
    my $title = $plugin->translate("'[_1]'(ID:[_2])' was posted.", $obj->title, $obj->id);

    # カスタムフィールドの項目を取得
    my $cf_meta = cf_meta( $app, $obj );
    my $edit_url = check_url($app, $obj, 'view');
    my $preview_url = check_url($app, $obj, 'preview_entry', $cf_meta);

    # 投稿通知メールテンプレートから本文を生成
    my $param = {};
    $obj->author_id( $author->id );
    $param->{entry} = $obj;
    $param->{author} = $author;
    $param->{edit_url} = $edit_url;
    $param->{preview_url} = $preview_url;
    $param->{blog} = $obj->blog;
    $param->{is_new} = 1 if !$app->param('id');
    if ( $app->param('crr') ) {
        $param->{crr} = 1;
        $param->{is_new} = 1 if $new_revision;
    }

    my $body = $app->build_email('new_entry_email', $param);

    # メールヘッダーの取得
    my $from = $app->config->EmailAddressMain || '';
    die $plugin->translate('EmailAddressMain is required.')
        unless $from;
    my %head = mail_head($app, $title, $from);
    return unless %head;

    # メール送信(ライターへのメール送信を抑制)
    #$emails{ $author->email } = 1;

    require MT::Mail;
    foreach ( keys %emails ) {
        $head{To} = $_;
        MT::Mail->send( \%head, $body )
            or $app->log( MT::Mail->errstr );
    }
}

sub cf_meta {
    my ($app, $obj) = @_;
    my $q = $app->param;
    my $blog_id = $app->param('blog_id') || 0;

    return {} if !$q->param('customfield_beacon');

    require CustomFields::Field;
    my $meta = {};
    foreach ($q->param()) {
        next if $_ eq 'customfield_beacon';
        if (m/^customfield_(.*?)$/) {
            my $field_name = $1;
            if ( m/^customfield_(.+?)_cb_beacon$/ ) {
                $field_name = $1;
                $meta->{$field_name} = defined( $q->param("customfield_$field_name") )
                  ? $q->param("customfield_$field_name")
                  : '0';
            }
            else {
                my $field = CustomFields::Field->load(
                  {
                    $blog_id ? ( blog_id => [ $blog_id, 0 ] ) : ( blog_id => $blog_id ),
                    basename => $field_name
                  }
                ) or next;
                if ($field->type eq 'datetime') {
                    $meta->{$field_name} =
                        $q->param("customfield_$field_name") ne ''
                        ? $q->param("customfield_$field_name")
                        : undef;
                } else {
                    $meta->{$field_name} =
                        $q->param("customfield_$field_name");
                }
            }
        }
    }
    $meta;
}


sub load_checkers {
    my ($checkers) = @_;
    die 'Invalid args. 1st args must be ARRAY referrence.'
        unless ref($checkers) eq 'ARRAY';
    my $authors = get_checkers();
    my $object_loop = [];
    foreach my $author ( @$authors ) {
        my $row = {
            label => $author->nickname,
            id => $author->id,
            checked => array_search($author->id, $checkers) ? 'checked' : ''
        };
        push @$object_loop, $row;
    }
    [ sort{ $b->{checked} cmp $a->{checked} } @$object_loop ];
}

sub get_crChangeBtnLabel {
    my ($type) = @_;
    my $function = <<'EOD';
    jQuery(document).ready(function(){
       crChangeBtnLabel();
    });
    function checked_any_authorities() {
        var checked_any = false;
        jQuery('.cr_check_item').each(function(){
            if ( jQuery(this).attr('checked') ) {
                checked_any = true;
            }
        });
        return checked_any;
    }
    function crChangeBtnLabel() {
        var checked_any = checked_any_authorities();
        <__trans_section component="CheckRelease">
        var save_label = '<__trans phrase="writer_save_text" />';
        var label = save_label;
        <mt:if name="mt_version_id" ge="5.1">
        var checked_label = '<mt:if name="created_by_is_me"><__trans phrase="writer_save_text_checked" /><mt:else><__trans phrase="writer_save_text" /></mt:if>';
        <mt:else>
        var checked_label = '<__trans phrase="writer_save_text_checked" />';
        </mt:if>
        var publish_label = '<mt:if name="is_replace"><__trans phrase="Replace"><mt:else><__trans phrase="publish" /></mt:if>';
        </__trans_section>

        var select_checker = <mt:var name="select_checker" />;
        var check_required = <mt:var name="check_required" />;
        var add_class = false;
        if ( checked_any || !select_checker || (select_checker && check_required) ) {
            label = checked_label;
            add_class = true;
        }
        var selectedStatus = jQuery('select#status').val();
        if ( selectedStatus == 2 ) {
            label = publish_label;
            add_class = false;
        } else if ( selectedStatus == 4 ) {
            label = save_label;
            add_class = false;
        }

        jQuery('#cr_draft_button_text').html(label);
        jQuery('#cr_draft_button_text').parent('button').each(function(){
        	jQuery(this).attr('title', label);
        	if ( add_class ) {
        		jQuery(this).addClass('check_status');
        	} else {
        		jQuery(this).removeClass('check_status');
        	}
        });
    }
    function crChackStatus() {
        if ( jQuery('#cr_draft_button_text').text() == '承認申請' ) {
            jQuery('input[name=check_status]').val(1);
        }
    }
EOD
    if ( $type eq 'writer' ) {
        $function .= q{
    <mt:unless name="cr_message">crChangeBtnLabel();</mt:unless>
};
    } elsif ( $type eq 'editor' ) {
        $function .= q{
    <mt:if name="created_by_is_me"><mt:unless name="cr_message">crChangeBtnLabel();</mt:unless></mt:if>
};
    }
    $function;
}

sub unable_plugin {
    my $lm = CheckRelease::Plugin->instance()->license_manager;
    return 1 if !$lm->is_registered && $lm->is_expired;
    return 1 unless has_setting();
    0;
}

sub has_setting {
    my $plugin = MT->component('CheckRelease');
    my $config = $plugin->get_config_hash();
    ref($config) ? 1 : 0;
}

1;
